# -*- coding: utf-8 -*-
import re
from zhon import hanzi


def is_ascii(c):
    '''Check if an character is in ASCII.

    >>> is_ascii('E')
    True
    >>> is_ascii('?')
    True
    >>> is_ascii('中')
    False
    >>> is_ascii('？')
    False
    '''
    return ord(c) < 128


def is_cjk(c):
    '''Check if an character is in CJK.
    >>> is_cjk('中')
    True
    >>> is_cjk('？')
    False
    >>> is_cjk('E')
    False
    >>> is_cjk('?')
    False
    '''
    return True if re.match('[{}]'.format(hanzi.characters), c) else False


def is_cjk_punctuation(c):
    '''Check if an character is a CJK punctuation.
    >>> is_cjk_punctuation('？')
    True
    >>> is_cjk_punctuation('?')
    False
    >>> is_cjk_punctuation('中')
    False
    >>> is_cjk_punctuation('E')
    False
    '''
    return True if c in hanzi.punctuation else False


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# Inspired by https://blog.csdn.net/ancale/article/details/27982553
from .utils import is_ascii, is_cjk, is_cjk_punctuation
from docutils.nodes import *


def setup(app):
    app.connect('doctree-resolved', process_chinese_paragraph)


def cjkspacing(text):
    '''Fix spacing issue related to newlines of CJK characters.

    The idea is to split input text by newlines and then join them to a string.
    '''
    # split input text into lines
    lines = text.splitlines()
    # deal with exceptions
    for i in range(1, len(lines)):  # start from 1
        # current line starts with ASCII and last line starts with hanzi
        try:
            if is_ascii(lines[i][0]) and not is_cjk_punctuation(lines[i-1][-1]):
                lines[i] = " " + lines[i]
            if is_ascii(lines[i][-1]) and not is_ascii(lines[i+1][0]):
                lines[i] = lines[i] + " "
        except IndexError:
            pass
    text = "".join(lines)
    return text


class ParagraphVisitor(NodeVisitor):
    def dispatch_visit(self, node):
        if isinstance(node, paragraph):
            for i in range(len(node.children)):
                if type(node[i]) == Text:
                    node[i] = Text(cjkspacing(node[i].astext()))


def process_chinese_paragraph(app, doctree, docname):
    pv = ParagraphVisitor(doctree)
    doctree.walk(pv)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open('README.rst') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='sphinx-cjkspace',
    version='0.1.1',
    description='Python extension to remove extra spaces between CJK characters',
    long_description=readme,
    author='Dongdong Tian',
    author_email='seisman.info@gmail.com',
    url='https://github.com/seisman/sphinx-cjkspace',
    license=license,
    packages=find_packages(exclude=('tests', 'docs')),
    install_requires=['sphinx', 'zhon'],
)
